from machine import Pin,SPI,PWM
import framebuf
import time

LCD_WIDTH = 172
LCD_HEIGHT = 320

MISO = 12
MOSI = 11
SCK = 10
LCD_CS = 9
LCD_DC = 14
LCD_RST = 13
LCD_BL = 15

# Pin definition
I2C_SDA = 6
I2C_SDL = 7
I2C_IRQ = 8
I2C_RST = 5


class lcd_jd9853(framebuf.FrameBuffer):
    def __init__(self): #SPI initialization  SPI初始化
        self.width = LCD_WIDTH
        self.height = LCD_HEIGHT
        
        self.cs = Pin(LCD_CS, Pin.OUT)
        self.rst = Pin(LCD_RST, Pin.OUT)
        self.dc = Pin(LCD_DC, Pin.OUT)
        
        self.cs(1)
        self.spi = SPI(1,100_000_000,polarity=0, phase=0,bits= 8,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)
        
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.init_display()
        
        #Define color, Micropython fixed to BRG format  定义颜色，Micropython固定为BRG格式
        self.red   =   0x07E0
        self.green =   0x001f
        self.blue  =   0xf800
        self.white =   0xffff
        self.black =   0x0000
        self.brown =   0X8430
        
        self.fill(self.white) #Clear screen  清屏
        self.show()#Show  显示

        self.pwm = PWM(Pin(LCD_BL))
        self.pwm.freq(5000) #Turn on the backlight  开背光
        
    def write_cmd(self, cmd): #Write command  写命令
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf): #Write data  写数据
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)
        
    def set_bl_pwm(self,duty): #Set screen brightness  设置屏幕亮度
        self.pwm.duty_u16(duty)#max 65535
        
    def init_display(self): #LCD initialization  LCD初始化
        self.rst(0)
        time.sleep_ms(100)
        self.rst(1)
        time.sleep_ms(10)
    
        self.write_cmd(0x11) 

        time.sleep_ms(120) 

        self.write_cmd(0xDF) 
        self.write_data(0x98) 
        self.write_data(0x53) 

        self.write_cmd(0xDF) 
        self.write_data(0x98) 
        self.write_data(0x53) 

        self.write_cmd(0xB2) 
        self.write_data(0x23) 

        self.write_cmd(0x36)   
        self.write_data(0x00)  

        self.write_cmd(0xB7) 
        self.write_data(0x00) 
        self.write_data(0x47) 
        self.write_data(0x00) 
        self.write_data(0x6F) 

        self.write_cmd(0xBB) 
        self.write_data(0x1C) 
        self.write_data(0x1A) 
        self.write_data(0x55) 
        self.write_data(0x73) 
        self.write_data(0x63) 
        self.write_data(0xF0) 

        self.write_cmd(0xC0) 
        self.write_data(0x44) 
        self.write_data(0xA4) 

        self.write_cmd(0xC1) 
        self.write_data(0x16) 

        self.write_cmd(0xC3) 
        self.write_data(0x7D) 
        self.write_data(0x07) 
        self.write_data(0x14) 
        self.write_data(0x06) 
        self.write_data(0xCF) 
        self.write_data(0x71) 
        self.write_data(0x72) 
        self.write_data(0x77) 

        self.write_cmd(0xC4) 
        self.write_data(0x00) 
        self.write_data(0x00) 
        self.write_data(0xA0) 
        self.write_data(0x79) 
        self.write_data(0x0B) 
        self.write_data(0x0A) 
        self.write_data(0x16) 
        self.write_data(0x79) 
        self.write_data(0x0B) 
        self.write_data(0x0A) 
        self.write_data(0x16) 
        self.write_data(0x82) 

        self.write_cmd(0xC8) 
        self.write_data(0x3F) 
        self.write_data(0x32) 
        self.write_data(0x29) 
        self.write_data(0x29) 
        self.write_data(0x27) 
        self.write_data(0x2B) 
        self.write_data(0x27) 
        self.write_data(0x28) 
        self.write_data(0x28) 
        self.write_data(0x26) 
        self.write_data(0x25) 
        self.write_data(0x17) 
        self.write_data(0x12) 
        self.write_data(0x0D) 
        self.write_data(0x04) 
        self.write_data(0x00) 
        self.write_data(0x3F) 
        self.write_data(0x32) 
        self.write_data(0x29) 
        self.write_data(0x29) 
        self.write_data(0x27) 
        self.write_data(0x2B) 
        self.write_data(0x27) 
        self.write_data(0x28) 
        self.write_data(0x28) 
        self.write_data(0x26) 
        self.write_data(0x25) 
        self.write_data(0x17) 
        self.write_data(0x12) 
        self.write_data(0x0D) 
        self.write_data(0x04) 
        self.write_data(0x00) 

        self.write_cmd(0xD0) 
        self.write_data(0x04) 
        self.write_data(0x06) 
        self.write_data(0x6B) 
        self.write_data(0x0F) 
        self.write_data(0x00) 

        self.write_cmd(0xD7) 
        self.write_data(0x00) 
        self.write_data(0x30) 

        self.write_cmd(0xE6) 
        self.write_data(0x14) 

        self.write_cmd(0xDE) 
        self.write_data(0x01) 

        self.write_cmd(0xB7) 
        self.write_data(0x03) 
        self.write_data(0x13) 
        self.write_data(0xEF) 
        self.write_data(0x35) 
        self.write_data(0x35) 

        self.write_cmd(0xC1) 
        self.write_data(0x14) 
        self.write_data(0x15) 
        self.write_data(0xC0) 

        self.write_cmd(0xC2) 
        self.write_data(0x06) 
        self.write_data(0x3A) 

        self.write_cmd(0xC4) 
        self.write_data(0x72) 
        self.write_data(0x12) 

        self.write_cmd(0xBE) 
        self.write_data(0x00) 

        self.write_cmd(0xDE) 
        self.write_data(0x02) 

        self.write_cmd(0xE5) 
        self.write_data(0x00) 
        self.write_data(0x02) 
        self.write_data(0x00) 

        self.write_cmd(0xE5) 
        self.write_data(0x01) 
        self.write_data(0x02) 
        self.write_data(0x00) 

        self.write_cmd(0xDE) 
        self.write_data(0x00) 

        self.write_cmd(0x35) 
        self.write_data(0x00) 

        self.write_cmd(0x3A) 
        self.write_data(0x05)  # 06=RGB666；05=RGB565

        self.write_cmd(0x2A) 
        self.write_data(0x00) 
        self.write_data(0x22) # Start_X=34
        self.write_data(0x00) 
        self.write_data(0xCD) # End_X=205

        self.write_cmd(0x2B) 
        self.write_data(0x00) 
        self.write_data(0x00) # /Start_Y=0
        self.write_data(0x01) 
        self.write_data(0x3F) # End_Y=319

        self.write_cmd(0xDE) 
        self.write_data(0x02) 

        self.write_cmd(0xE5) 
        self.write_data(0x00) 
        self.write_data(0x02) 
        self.write_data(0x00) 

        self.write_cmd(0xDE) 
        self.write_data(0x00) 

        # self.write_cmd(0xC2) 
        # self.write_data(0x08) 
        self.write_cmd(0x21) 
        self.write_cmd(0x29) 

        time.sleep_ms(10) 
    
    #设置窗口    
    def setWindows(self,Xstart,Ystart,Xend,Yend):
        Xstart = Xstart + 34 
        Xend = Xend + 34 
        self.write_cmd(0x2A)
        self.write_data(Xstart >> 8)
        self.write_data(Xstart)
        self.write_data((Xend-1) >> 8)
        self.write_data(Xend-1)
        
        self.write_cmd(0x2B)
        self.write_data((Ystart) >> 8)
        self.write_data(Ystart)
        self.write_data(((Yend)-1) >> 8)
        self.write_data((Yend)-1)
        
        self.write_cmd(0x2C)
     
    #Show  显示   
    def show(self): 
        self.setWindows(0,0,self.width,self.height)
        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)
        
    
    def Windows_show(self, Xstart, Ystart, Xend, Yend):
        if Xstart > Xend:
            data = Xstart
            Xstart = Xend
            Xend = data
            
        if (Ystart > Yend):        
            data = Ystart
            Ystart = Yend
            Yend = data
        
        if (Xend < self.width):
            Xend = Xend + 1
        if (Yend < self.height):
            Yend = Yend + 1
            
        self.setWindows(Xstart,Ystart,Xend,Yend)      
        self.cs(1)
        self.dc(1)
        self.cs(0)
        for i in range (Ystart,Yend):             
            Addr = (Xstart * 2) + (i * 240 * 2)                
            self.spi.write(self.buffer[Addr : Addr+((Xend-Xstart)*2)])
        self.cs(1)
        
    #Write characters, size is the font size, the minimum is 1  
    #写字符，size为字体大小,最小为1
    def write_text(self,text,x,y,size,color):
        ''' Method to write Text on OLED/LCD Displays
            with a variable font size

            Args:
                text: the string of chars to be displayed
                x: x co-ordinate of starting position
                y: y co-ordinate of starting position
                size: font size of text
                color: color of text to be displayed
        '''
        background = self.pixel(x,y)
        info = []
        # Creating reference charaters to read their values
        self.text(text,x,y,color)
        for i in range(x,x+(8*len(text))):
            for j in range(y,y+8):
                # Fetching amd saving details of pixels, such as
                # x co-ordinate, y co-ordinate, and color of the pixel
                px_color = self.pixel(i,j)
                info.append((i,j,px_color)) if px_color == color else None
        # Clearing the reference characters from the screen
        self.text(text,x,y,background)
        # Writing the custom-sized font characters on screen
        for px_info in info:
            self.fill_rect(size*px_info[0] - (size-1)*x , size*px_info[1] - (size-1)*y, size, size, px_info[2]) 
        
        
        
        

class touch_axs5106l:
    def __init__(self, device_addr=0x63, mode=0, i2c_num=1, i2c_sda=I2C_SDA, i2c_scl=I2C_SDL, irq_pin=I2C_IRQ, rst_pin=I2C_RST):
        self.bus = machine.I2C(id=i2c_num, scl=machine.Pin(i2c_scl), sda=machine.Pin(i2c_sda), freq=400_000)
        self.device_addr = device_addr  # Set slave address 
        self.int = machine.Pin(irq_pin, machine.Pin.IN, machine.Pin.PULL_UP)
        self.rst = machine.Pin(rst_pin, machine.Pin.OUT)

        self.point_count = 0
        self.coordinates = [{"x": 0, "y": 0} for _ in range(1)]
        self.reset()
        self.read_flag = True
        
        self.int.irq(handler = self.int_cb, trigger = machine.Pin.IRQ_FALLING)

    def int_cb(self, pin):
        self.read_touch_data()
    
    def reset(self):
        self.rst(1)
        time.sleep_ms(10)
        self.rst(0)
        time.sleep_ms(100)      
        self.rst(1)
        time.sleep_ms(100) 

    def write_cmd(self, reg_addr, data):
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
    def read_bytes(self, reg_addr, length):
        try:
        
            self.bus.writeto(int(self.device_addr), bytes([reg_addr]))
             
            rec = self.bus.readfrom(int(self.device_addr), length)
            return rec
        except Exception as e:
            print(f"Error reading bytes: {e}")
            return None

    def read_touch_data(self):
        AXS5106L_TOUCH_DATA_REG = 0x01
        buf = self.read_bytes(AXS5106L_TOUCH_DATA_REG, 14)
        if buf is not None and buf[1]!= 0:
            self.point_count = buf[1]
            
            if buf is not None:
                for i in range(self.point_count):
                    self.coordinates[i]["x"] = ((buf[2 + i * 6] & 0x0f) << 8) + (buf[3 + i * 6])
                    self.coordinates[i]["x"] = LCD_WIDTH - 1 - self.coordinates[i]["x"]
                    self.coordinates[i]["y"] = ((buf[4 + i * 6] & 0x0f) << 8) + (buf[5 + i * 6])

    def get_touch_xy(self):
        if self.point_count != 0:
            self.point_count = 0
            return self.coordinates
        else:
            return None

if __name__ == '__main__':
    lcd = lcd_jd9853()
    lcd.set_bl_pwm(30000)
    # lcd.lcd_fill(0xffff)
    # time.sleep(1)
    # lcd.lcd_fill(0xf800)
    # time.sleep(1)
    # lcd.lcd_fill(0x07e0)
    # time.sleep(1)
    # lcd.lcd_fill(0x001f)
    # time.sleep(1)
    # lcd.lcd_fill(0xffff)
    
    lcd.fill(lcd.red)
    lcd.show()
    time.sleep(1)
    lcd.fill(lcd.green)
    lcd.show()
    time.sleep(1)
    lcd.fill(lcd.blue)
    lcd.show()
    time.sleep(1)
    
    lcd.fill(lcd.white)
    lcd.show()
    touch = touch_axs5106l()
    while True:
        touch.read_touch_data()
        coordinates = touch.get_touch_xy()
        if coordinates is not None:
            for i, coord in enumerate(coordinates):
                lcd.rect(coord['x'], coord['y'], 4, 4, lcd.red)
                lcd.show()
                print(f"Point {i + 1} coordinates: x={coord['x']}, y={coord['y']}")
        time.sleep_ms(10)